/*
 * File : DlDisconnected.C
 *
 * Description : This file contains method descriptions of the
 *	dlDisconnected class.
 */

#include "dlInSetupPending.h"
#include "dlOutSetupPending.h"
#include "dlDisconnected.h"
#include "phEvents.h"
#include "dlEvents.h"
#include "frame.h"
#include "dllayer.h"

/*
 * Function : dlDisconnected()
 *
 * Description : This is the constructor. Calls base class
 *	constructor, and Resets state variables of the data-link
 *	layer.
 */

dlDisconnected::dlDisconnected(dllayer *dl) : dlStateMachine(dl) 
{
	_dl->ResetStateVariables();
}
/*
 * Function : DlConnectReq()
 *
 * Descrition : This method handles the DL_ConnectReq event. A
 *	SABM is transmitted, and state changed to dlOutSetupPending.
 */

dlStateMachine *
dlDisconnected::DlConnectReq(int &errorCode)
{
	TRACE(dlSmModule,1,("Entering DlConnectReq _dl : %x\n", _dl));

	// now, go ahead and send a SABM
	_dl->XmitFrame(errorCode,TRUE, SABM,1);

	if ( errorCode == codeSuccessful ) {		
		_dl->StartT1Timer(FALSE); // start a new timer					
		_dl->_setupType = initial; // setup type is initial
		CHANGE_STATE(dlOutSetupPending);
	}
	else 
		return this;
}

/*
 * Function : DlHandleIframe()
 *
 * Description : Handles the IFRAME event. Transmits a DM. 
 *	No state change.
 */

dlStateMachine *
dlDisconnected::HandleIframe(int &errorCode)
{
	if ( _dl->_lastFrameRecd->GetPf()) // If P=1
		_dl->XmitFrame(errorCode,FALSE,DM,1);
	return this;
}
/*
 * Function : HandleSabm()
 *
 * Description : Handles an incoming connection request in the
 *	form of a SABM. Invokes DL_ConnectInd on the network layer.
 *	Changes state to dlInSetupPending.
 */

dlStateMachine *
dlDisconnected::HandleSabm(int &errorCode)
{
	TRACE(dlSmModule,1,("Entering HandleSabm: _dl : %x\n",_dl));
	_dl->_nPlus1->Command(DL_ConnectInd,errorCode);
	_dl->_setupType = initial; // Setup type is initial
	CHANGE_STATE(dlInSetupPending);
}
/*
 * Function : HandleDisc()
 *
 * Description : Handles a DISC command event. Transmits a DM 
 *	Remains in the same state.
 */

dlStateMachine *
dlDisconnected::HandleDisc(int &errorCode)
{
	// Transmit a DM with F=P
	_dl->XmitFrame(errorCode,FALSE,DM,_dl->_lastFrameRecd->GetPf());
	return this;
}
 
/*
 * Function : HandleRr()
 *
 * Description : Handles a RR event. Transmits a DM.
 *	Remains in the same state.
 */

dlStateMachine *
dlDisconnected::HandleRr(int &errorCode)
{
	// Transmit a DM if RR is a command and P=1
	if ( _dl->_lastFrameRecd->IsCmd() && _dl->_lastFrameRecd->GetPf())
		_dl->XmitFrame(errorCode,FALSE,DM,1);
	return this;
}
 
/*
 * Function : HandleRnr()
 *
 * Description : Handles a RNR event. Transmits a DM.
 *	Remains in the same state.
 */

dlStateMachine *
dlDisconnected::HandleRnr(int &errorCode)
{
	// Transmit a DM if RNR is a command and P=1
	if ( _dl->_lastFrameRecd->IsCmd() && _dl->_lastFrameRecd->GetPf())
		_dl->XmitFrame(errorCode,FALSE,DM,1);
	return this;
}
 
/*
 * Function : HandleRej()
 *
 * Description : Handles a REJ event. Transmits a DM.
 *	Remains in the same state.
 */

dlStateMachine *
dlDisconnected::HandleRej(int &errorCode)
{
	// Transmit a DM if REJ is a command and P=1
	if ( _dl->_lastFrameRecd->IsCmd() && _dl->_lastFrameRecd->GetPf())
		_dl->XmitFrame(errorCode,FALSE,DM,1);
	return this;
}

/*
 * Function : PrintState
 *
 * Description : Prints state.
 */

void 
dlDisconnected::PrintState() 
{
	TRACE(dlSmModule,2,("State Disconnected\n"));
}


