/*
 * File : phStateMachine
 *
 * Description : This file contains the class declaration for
 *		phStateMachine.
 */

#ifndef _phStateMachine_h_
#define _phStateMachine_h_

#include "trace.h"
#include "errorCodes.h"

class phlayer;
class layer;

/*
 * Class : phStateMachine
 *
 * Description : This is the base class for the state machine classes
 *	for the physical layer. Sub-classes will redefine various methods
 *	to provide functionality to handle the various events.
 *
 * Attributes:
 *	phlayer *_ph : Pointer to the physical layer which created this
 *		state machine object.
 */

class phStateMachine {
 public:
	phStateMachine(phlayer *ph) : _ph(ph) {};
	virtual ~phStateMachine() {}; 

	virtual phStateMachine *PhActivateReq(int &errorCode)
		{ errorCode = codeSuccessful; return this; }
	virtual phStateMachine *PhDataReq(int &errorCode)
		{ errorCode = codeSuccessful; return this; }
	virtual phStateMachine *PhDeactivateReq(int &errorCode)
		{ errorCode = codeSuccessful; return this; }

	virtual phStateMachine *PhActivate(int &errorCode)
		{ errorCode = codeSuccessful; return this; }
	virtual phStateMachine *PhData(int &errorCode)
		{ errorCode = codeSuccessful; return this; }
	virtual phStateMachine *PhDeactivate(int &errorCode)
		{ errorCode = codeSuccessful; return this; }
	virtual void PrintState() = 0;
 protected:
	phlayer *_ph;
};

DECLARE_TRACE_MODULE(phSmModule);

#endif
