/*
 * File : timerHandler.h
 *
 * Description : This file declares the timerHandler class.
 */

#ifndef _timerHandler_h_
#define _timerHandler_h_

#include "types.h"
#include "eventHandler.h"

/*
 * Class : timerHandler
 *
 * Description : This class inherits from the eventHandler base
 * class, and provides the functionality to create timers,
 * and handle timeout events. SetEvent and CheckEvent are
 * redefined.
 *
 * Attributes:
 *	timeval _nextto: The time when this timer will timeout. 
 *	int	_xo : The value in seconds of this timer.
 */

class timerHandler : public eventHandler {

 public:

	timerHandler(int timeout);
	virtual ~timerHandler();
	void Print();
	virtual void EventCallback();
	void Disable();

	void SetEvent(fd_set *readfds,
		fd_set *writefds, fd_set *exceptfds,
		struct timeval *timeout);

	virtual void CheckEvent(fd_set *readfds,
		fd_set *writefds, fd_set *exceptfds,
		struct timeval *curtime);

 private:
	Boolean _disable;
	timeval	_nextto;
	int	_xo;
};

#endif
